/*
Script: NWR-radios.js
Purpose: handle page formatting for the NWR-radios selection/display
Author: Ken True - Saratoga-weather.org

This script is based on scripts orginally developed by
Chappellweather.com - http://chappellweather.com/
with mods by:
Clifton Virginia Weather - http://cliftonvaweather.com/
and by SE Lincoln Weather - http://www.gwwilkins.org/

Major changes from the Chappell/Clifton/Lincoln versions include:
- using NWR-radios-data.js from noaaweatherradio.org for data
- using the data feed to generate the dropdown station selector HTML
- eliminating the AJAX fetching of data/dropdown files 
- adding a generalized selection of default station based on call/location/attribution/website 

// Version 1.00 - 26-Jun-2017 - initial release
// Version 1.01 - 29-Jun-2017 - added autoplaystartup flag
// Version 1.04 - 05-Jul-2017 - corrected attributions

*/
// note: all the data will come from NWR-radio-data.js JSON loaded by the HTML page
var selectedstation = '';

function loadDropdown(data,findstation) {
	// generate the option/select box HTML from the JSON file
/*
    "KEC49": {
        "call": "KEC49",
        "alt": "N",
        "loc": "Monterey, CA",
        "freq": "162.550 MHz",
        "who": "Saratoga-Weather.org",
        "type": "PWS",
        "wxurl": "http:\/\/saratoga-weather.org",
        "lat": 37.155,
        "long": -121.897222,
        "watts": "330",
        "xmloc": "Mt. Umunhum,CA",
        "wfo": "Monterey",
        "wxown": "KEC49",
        "radiourl": "http:\/\/saratogawx.dyndns.org:88\/broadwave.mp3",
        "mapurl": "http:\/\/www.nws.noaa.gov\/nwr\/Maps\/GIF\/KEC49.gif",
        "logo": "http:\/\/noaaweatherradio.org\/content\/thumbnails\/NWR150.png",
        "state": "California",
        "stateabbr": "CA"
    },
*/	
	var keylist = Object;
	var out = '<p>No List Available</p>';
	var re = RegExp(findstation,'i');
	
	for (var call in data) {
		
		var nkey = data[call].state+'|'+data[call].loc+'|'+data[call].call;
		keylist[nkey] = call;
	}
	
	var skeys = Object.keys(keylist).sort();

  var laststate = '';
	var firstcall = '';
	out = "<select>\n";
	for (var i = 0; i < skeys.length; i++) {
//			console.log('i='+i+' skeys="'+skeys[i]+'"');
			var key = skeys[i];
			var kp = key.split("|");
//      console.log('kp="'+kp+'"');
			
	  var state = kp[0];
		var city  = kp[1];
		var call  = kp[2];
		if( typeof data[call] == "undefined" ) { break; }
		var callx = data[call].wxown;
		var sabbr = data[call].stateabbr;
		if(firstcall.length == 0) {firstcall = call;}
			
		if(sabbr == "na") {
			city = data[call].who;
			data[call].loc = '';
			callx = '';
		}

		if( laststate !== state) {
			if(laststate.length > 0) {
				out += "</optgroup>\n";
			}
			out += '<optgroup label="'+state+"\">\n";
			laststate = state;
		}
		var sel = '';
		if(selectedstation.length == 0) {
			if(re.test(call) || 
			   re.test(city) ||
			   re.test(data[call].who) ||
				 re.test(data[call].wxurl) ) { // is this the one we're looking for?
			     //console.log('selectedstation="'+selectedstation+'" length='+
			     //  selectedstation.length+' call="'+call+'"');
			     sel = " selected=\"selected\"";
			     selectedstation = call;
			 }
		}
		out += "<option value=\""+call+"\""+sel+">"+callx;
//		if(call !== data[call].wxown) {
	  if(data[call].alt == 'Y') {
			out += " Alternate";
		}
		out += " - "+city+"</option>\n";
	
	}
	out += "<optgroup>\n</select>\n";
	
	if(selectedstation.length == 0) {
		selectedstation = firstcall;
		console.log('Note: startup="'+startup+
		  '" not found in call/loc/website, first station "'+selectedstation+'" used.');
	}
	
  return out;
}

function loadInfo() {

	$("#stationcount").html('(' + data['radiocount'] + ' Stations, ' + data['stationcount'] + ' Streams)');
	$("#update").html(data['datadate']);
	var ddlist = loadDropdown(data,startup);
	$('#dropdown').html(ddlist);
	if(streamprovider) {
		$('#provider').html('We are pleased to provide streaming audio for our area from our systems.');
	}
}

function showStation(call, auto) {
    var rl = data[call].radiourl;
    if(auto) {
        var txt = '<audio id="aud" controls autoplay src="' + rl + '" type="audio/mpeg">\n' +
        'Your browser does not support the audio tag.\n</audio>';
    } else {
        var txt = '<audio id="aud" controls src="' + rl + '" type="audio/mpeg" >\n' +
        'Your browser does not support the audio tag.\n</audio>';
    }
    $('#radio').html(txt);
    $('#station').html(data[call].call);
    $('#freq').html(data[call].freq);
    $('#locate').html(data[call].loc);
		$('#xmloc').html(data[call].xmloc);
    if(data[call].wxurl == "") {
        var txt = data[call].who;
    } else {
        var txt = '<a href="' + data[call].wxurl + '" target="_blank">' + data[call].who + '</a>';
    }
    $('#provide').html(txt);
    var txt = '<img src="' + data[call].logo + '" height="50 width="50" alt="" />'
    $('#logo').html(txt);
    var txt = '<img src="' + data[call].mapurl + '" class="img-responsive" style="padding:5%;" alt="Coverage Map - ' + data[call].loc + ' Not Available" />'
    $('#cmap').html(txt);
    var txt = data[call].loc + ' NOAA Radio ' + data[call].call + ' Coverage Area';
    $('#maphead').html(txt);
    showmap(call);
    setTimeout(function(){checkNetwork();},10000);
}

function loadPointer() {
    $("select").change(function () {
        var call = $(this).val();
        showStation(call, true);
    });
}

function checkNetwork() {
    var x =$('audio')[0];
    var e = x.networkState;
    if(e != null && e == 3){
        var txt = '<img src="redx.png" height="20 width="20" alt="" />&nbsp;&nbsp;Failed To Load Audio Stream';
        $('#radio').html(txt);
    }
}

function showmap(call) {
    if(call.indexOf('-') > -1) {
        call = call.split('-')[0];
    }
    if(data[call] == null) {
        $('#map_container').css('height', '75px');
        var txt = '<h2 style="color: red">Transmiter Location Map Unavailable For: ' + call + '</h2>';
        $('#map').html(txt);
        return;
    } else {
        $('#map_container').css('height', '500px');
    }
    var content = '';
    content = '<div id="content">\n'+
        '<div id="siteNotice">\n'+
        '</div>\n'+
        '<h3 id="firstHeading" class="firstHeading">NWR Station: '+ call +'</h3>\n'+
        '<h4>' + data[call].loc +  '</h4>\n'+
        '<h4>Sited at ' + data[call].xmloc +  '</h4>\n'+
        '<div id="bodyContent">\n'+
        '<table>\n' +
        '<tr><td><b>Latitude:</b></td><td>' + data[call].lat + '</td></tr>\n'+
        '<tr><td><b>Longitude:</b></td><td>' + data[call].long + '</td></tr>\n'+
        '</table>\n' +
        '</div>\n'+
        '</div>\n';

    var xmitr = {
        url: 'xmit2.gif',
        size: new google.maps.Size(35, 35),
        origin: new google.maps.Point(0, 0),
        anchor: new google.maps.Point(0, 0),
    };
    var map = new google.maps.Map(document.getElementById('map'), {
        zoom: 10,
				scaleControl: true,
	      scrollwheel: false,
        center: {lat: data[call].lat, lng: data[call].long},
        mapTypeId: mapType
    });
    var marker = new google.maps.Marker({
            position: {lat: data[call].lat, lng: data[call].long},
            map: map,
            optimized:false,
            animation: google.maps.Animation.DROP,
            icon: xmitr,
            title: call + ' - Click for details',
            zIndex: 0
    });
    var infowindow = new google.maps.InfoWindow();
            google.maps.event.addListener(marker,'click', (function(marker,content,infowindow){
            return function() {
                infowindow.setContent(content);
                infowindow.open(map,marker);
                lastInfoWindow = infowindow;
                google.maps.event.addListener(map,'click', function(){
                    infowindow.close();
                    map.setCenter({lat: data[call].lat, lng: data[call].long});
                });
            };
    })(marker,content,infowindow));
}


$(document).ready(function () {
    loadInfo();
		
    setTimeout(function(){loadPointer();},1000);
    setTimeout(function(){showStation(selectedstation, autoplaystartup);},2000);
})
// end of NWR-radios.js script